import json
import math
from aheadworks_core.model.http.api_request import ApiRequest as Api
from typing import Optional


class BitbucketApiManager:
    """api manager for bitbucket"""

    def __init__(self, bitbucket_config):
        self.sorted_deployments = None
        self.sorted_pipelines = None
        self.config = bitbucket_config
        self.request = Api(config=self.config)

    def get_build_by_commit(self, commit_hash: str, pipe_name, depth=1):
        """
        :param commit_hash: str
        :param pipe_name: bitbucket pipeline name
        :param depth: search depth in parent commits
        :return: build number: int
        :raises Exception: Сommit hash should not be less than 6 symbols.
        """

        if len(commit_hash) < 6:
            raise Exception('Сommit hash should not be less than 6 symbols.')

        build = self.get_build_number(commit_hash=commit_hash, pipe_name=pipe_name)
        if build is None:
            if depth == 0:
                raise Exception('Build not found.')
            else:
                commit = self.get_commit(commit_hash=commit_hash)
                parent_commits = commit['parents']
                if len(parent_commits) > 1:
                    sorted_parent_commits = list()
                    for _ in parent_commits:
                        commit_hash = _['hash']
                        commit = self.get_commit(commit_hash=commit_hash)
                        sorted_parent_commits.append(commit)
                    sorted_parent_commits = sorted(sorted_parent_commits, key=lambda k: (k['date']), reverse=True)
                    parent_commit = sorted_parent_commits[0]['hash']
                else:
                    parent_commit = parent_commits[0]['hash']
            depth -= 1
            return self.get_build_by_commit(parent_commit, pipe_name, depth)

        return build

    def get_build_number(self, commit_hash: str, pipe_name):
        """get bitbucket bld number by commit

        :param commit_hash: bitbucket commit hash, full or short
        :param pipe_name: bitbucket pipeline name
        :return: build number
        """
        build_number = None
        pipelines = self.get_sorted_pipelines()

        for pipeline in pipelines['values']:
            pipeline_commit_hash: Optional[str] = pipeline['target']['commit']['hash']
            if pipeline_commit_hash.find(commit_hash) == 0:
                if 'result' in pipeline['state'] \
                        and pipeline['state']['result']['name'] == 'SUCCESSFUL' \
                        and pipeline['target']['selector']['pattern'] in pipe_name:
                    build_number = pipeline['build_number']
                    break

        return build_number

    def get_build_number_from_deploy(self, commit_hash: str):
        """get bitbucket bld number by commit

        :param commit_hash: bitbucket commit hash, full or short
        :return: build number
        """
        build_number = None
        deploys = self.get_sorted_deployments()

        for deploy in deploys['values']:
            pipeline_uuid = deploy['deployable']['pipeline']['uuid']
            if deploy['release']['commit']['hash'].find(commit_hash) == 0:
                pipeline = self.get_pipeline(pipeline_uuid)
                pipeline_commit_hash: Optional[str] = pipeline['target']['commit']['hash']
                if pipeline_commit_hash.find(commit_hash) == 0:
                    build_number = pipeline['build_number']
                    break

        return build_number

    def get_commit(self, commit_hash: str, config=None):
        if config is None:
            config = self.config
        commit = self.request.get(location='/2.0/repositories/{}/{}/commit/{}'.format(
            config.bitbucket_workspace,
            config.bitbucket_repo_slug,
            commit_hash))
        return json.loads(commit)

    def get_pipeline(self, pipeline_uuid):
        pipeline = self.request.get(location='/2.0/repositories/{}/{}/pipelines/{}'.format(
            self.config.bitbucket_workspace,
            self.config.bitbucket_repo_slug,
            pipeline_uuid))
        return json.loads(pipeline)

    def get_sorted_pipelines(self, params=None):
        if self.sorted_pipelines is None:
            self.sorted_pipelines = self.get_pipelines(params)

            values = self.sorted_pipelines['values']
            values.reverse()
            self.sorted_pipelines['values'] = values

        return self.sorted_pipelines

    def get_pipelines(self, params=None):
        pagelen_default = 100
        if params is None:
            params = {}
        params['pagelen'] = pagelen_default
        deploys = self.request.get(location='/2.0/repositories/{}/{}/pipelines/'.format(
            self.config.bitbucket_workspace,
            self.config.bitbucket_repo_slug
            ), params=params
        )
        result = json.loads(deploys)
        cur_page = result['page']
        count_page = math.ceil(result['size'] / pagelen_default)
        if cur_page < count_page:
            params['page'] = cur_page + 1
            result['values'].extend(self.get_pipelines(params=params)['values'])

        return result

    def get_sorted_deployments(self, params=None):
        if self.sorted_deployments is None:
            self.sorted_deployments = self.get_deployments(params)

            values = self.sorted_deployments['values']
            values = sorted(values, key=lambda k: (k['deployable']['created_on']), reverse=True)
            self.sorted_deployments['values'] = values

        return self.sorted_deployments

    def get_deployments(self, params=None):
        if params is None:
            params = {'pagelen': '100'}
        deploys = self.request.get(location='/2.0/repositories/{}/{}/deployments/'.format(
            self.config.bitbucket_workspace,
            self.config.bitbucket_repo_slug
            ), params=params
        )
        result = json.loads(deploys)
        if 'next' in result.keys():
            query = result['next'].split('?')[1]
            params = {}
            for _ in query.split('&'):
                params[_.split('=')[0]] = _.split('=')[1]
            result['values'].extend(self.get_deployments(params=params)['values'])

        return result

    def upload_artifacts(self, artifacts):
        responses = []
        if type(artifacts) is not list:
            artifacts = [artifacts, ]
        for f in artifacts:
            files = {'files': open(f, 'rb')}
            responses.append(
                self.request.post(
                    location = f"/2.0/repositories/{self.config.bitbucket_workspace}/{self.config.bitbucket_repo_slug}/downloads",
                    files=files
                )
            )
        return responses

    def get_repositories(self, name_contains = None):
        repopage = f"/2.0/repositories/{self.config.bitbucket_workspace}?pagelen=100&fields=next,values.links.clone.href"
        if name_contains:
            repopage = f'{repopage}&q=name~"{name_contains}"'
        full_repo_list = []
        while repopage is not None:
            response_text = self.request.get(location=repopage)
            page_json = json.loads(response_text)
            for repo in page_json['values']:
                repogit=repo['links']['clone'][1]['href']
                full_repo_list.append(repogit)
            repopage = page_json.get('next', None)
            if repopage:
                repopage = repopage.removeprefix('https://api.bitbucket.org')
        return full_repo_list

    def run_pipeline(self, repo, data):
        return self.request.post(
            f"/2.0/repositories/{repo}/pipelines/", json=data
        )



import requests
from requests.auth import HTTPBasicAuth

class ApiRequest:
    """request class for web api"""

    def __init__(self, config):
        self.config = config
        self.url = self.config.url

        if hasattr(self.config, 'auth_type') and self.config.auth_type == 'none':
            self.auth_headers = {}
        elif hasattr(self.config, 'auth_type') and self.config.auth_type == 'auth':
            self.auth = HTTPBasicAuth(self.config.user, self.config.token)
        else:
            self.auth_headers = {'Authorization': "Bearer {}".format(self.config.token), }

    def request(self, location, params=None, headers=None, method='GET', files=None):
        if headers is None and hasattr(self, 'auth_headers'):
            headers = self.auth_headers

        url = f'{self.url}{location}'
        r = requests.request(method=method, url=url, params=params, headers=headers, auth=self.auth, files=files)
        return r.text

    def get(self, location, params=None, headers=None):
        """ get request to bitbucket api
        :param location: url location path
        :param params: request query params
        :param headers: http headers
        :return: text of response
        """
        if headers is None:
            headers = self.auth_headers
        url = f'{self.url}{location}'
        r = requests.get(url=url, params=params, headers=headers)
        return r.text

    def post(self, location, headers=None, data=None, json=None, files=None):
        """
        :param location: location: url location path
        :param headers:  http headers
        :param data: request data
        :param json: json in request body
        :param files: dictionary with opened files
        :return: response
        """
        if headers is None:
            headers = self.auth_headers
        url = f'{self.url}{location}'
        r = requests.post(url, headers=headers, data=data, json=json, files=files)
        return r

    def put(self, location, headers=None, data=None, json=None):
        """
        :param location: url location path
        :param headers: http headers
        :param data: request data
        :param json: json in request body
        :return: response
        """
        url = f'{self.url}{location}'
        r = requests.post(url, headers=headers, data=data, json=json)
        return r

    def delete(self, location, headers=None):
        """
        :param location:  url location path
        :param headers: http headers
        :return: status code
        """
        url = f'{self.url}{location}'
        if headers is None:
            headers = self.auth_headers
        r = requests.delete(url, headers=headers)
        return r.status_code



from aheadworks_bitbucket_manager.api.bitbucket_api_manager import BitbucketApiManager
from aheadworks_bitbucket_manager.model.data.bitbucket import BitbucketConfig
from aheadworks_core.api.composer_manager import ComposerManager
from aheadworks_core.api.jira_api_manager import JiraApiManager
from aheadworks_core.api.discord_api_manager import DiscordApiManager
from aheadworks_core.api.magento_manager import MagentoManager
from aheadworks_core.api.file_manager import FileManager
from aheadworks_core.model.parser.json import Json as JsonParser
from aheadworks_core.model.http.api_request import ApiRequest
from aheadworks_core.model.data.data_object import DataObject
from datetime import datetime
from urllib.parse import urlparse
import boto3
import json
import os
import re
import shutil


class ReleaseManager:
    """api manager for release"""

    RELEASE_PACK_TASK_LABEL = 'RELEASE-PACK'
    PD_TASK_LABEL = 'PD'
    TEST_TASK_LABEL = 'TEST'

    def __init__(self, jira_api_config):
        self.jira_api_config = jira_api_config
        self.jira_api_manager = JiraApiManager(config=self.jira_api_config)
        self.discord_api_manager = DiscordApiManager()
        self.magento_manager = MagentoManager()
        self.file_manager = FileManager()
        self.json_parser = JsonParser()
        self.aws_s3 = boto3.resource('s3')

    def jira_release(self, jira_project_key, composer_file, discord_bot_url, path_to_files, assign_to):
        module_version = self.json_parser.get_variable_from_file('version', composer_file)
        # module_version = self.magento_manager.get_module_version(path_to_module)

        print(f"jira project key: {jira_project_key}")
        print(f"module version: ' {module_version}")
        print(f"discord bot url: {discord_bot_url}")
        print(f"path to files: {path_to_files}")
        print(f"assign to, account id: {assign_to}")

        if not jira_project_key:
            print('jira_project_key is empty, skip jira release.')
            return False

        jira_instance = self.jira_api_manager.get_jira_instance()

        files_to_upload = []
        file_names = os.listdir(path_to_files)
        for file_name in file_names:
            files_to_upload.append(('file', (file_name, open(f"{path_to_files}/{file_name}", 'rb'))))

        jql = 'labels%3D{}-{}'.format(jira_project_key, module_version)
        links = self.get_release_tasks(jql)

        release_task_key = links[self.RELEASE_PACK_TASK_LABEL]['key']
        pd_task_key = links[self.PD_TASK_LABEL]['key']
        test_task_key = links[self.TEST_TASK_LABEL]['key']

        # add attachments
        self.add_attachments_to_task(release_task_key, files_to_upload)

        # module_dependencies = self.magento_manager.get_module_dependencies(path_to_module)
        module_dependencies = self.magento_manager.get_module_dependencies_from_composer(composer_file)
        composer_package_name = ','.join(list(map(lambda x: x['full_module_name'], module_dependencies.values())))

        # assign release pack to user
        release_issue = jira_instance.issue(release_task_key)
        release_issue.update(assignee={'accountId': assign_to})
        jira_instance.add_comment(release_issue, f'Composer Package Name:\n{composer_package_name}')

        # uncomment if needed check transitions for issue
        # transitions = jira_instance.transitions(release_issue)
        # set done to pd and test issue. transition=31 - Task Done
        pd_issue = jira_instance.issue(pd_task_key)
        jira_instance.transition_issue(pd_issue, transition=31)

        test_issue = jira_instance.issue(test_task_key)
        jira_instance.transition_issue(test_issue, transition=31)

        # release current version
        version = jira_instance.get_project_version_by_name(jira_project_key, module_version)
        release_date = datetime.today().strftime('%Y-%m-%d')
        version.update(released=True, releaseDate=release_date)

        project = jira_instance.project(jira_project_key)

        msg = '{} {}\n'.format(project.name, module_version)
        msg += '\n'.join(list(map(lambda x: x['url'], links.values())))
        msg += '\n' + self.jira_api_manager.get_release_report_all_issues_url(jira_project_key, version.id)

        self.discord_api_manager.send_msg(discord_bot_url, msg)

        return True

    def get_release_tasks(self, jql):
        print(f" https://aheadworks.atlassian.net/issues/?jql={jql}")
        links = dict()
        search_labels = [self.RELEASE_PACK_TASK_LABEL, self.PD_TASK_LABEL, self.TEST_TASK_LABEL]
        tasks = self.jira_api_manager.search_tasks_jql(jql)
        if 'issues' in tasks and len(tasks['issues']):
            for task in tasks['issues']:
                if 'labels' in task['fields']:
                    task_labels = task['fields']['labels']
                    label_intersection = list(set(task_labels) & set(search_labels))
                    if len(label_intersection) == 1:
                        task_key = task['key']
                        task_url = self.jira_api_manager.get_issue_url(task_key)
                        links[label_intersection[0]] = dict({'url': task_url, 'key': task_key})
                    else:
                        raise Exception('Incorrect release labels count.')
                else:
                    raise Exception('Tasks data returned by JIRA has no data on labels')
        else:
            print(tasks)
            raise Exception('Release Tasks not found.')

        return links

    def add_attachments_to_task(self, task_key, files):
        tasks = self.jira_api_manager.add_attachments_to_task(task_key, files)

    def build_swagger_web_api_doc(
            self,
            path_to_module,
            magento_url,
            magento_path_on_server='/var/www/html',
            ssh_port=22,
            ssh_user='root',
            ssh_pass='root'
    ):
        obj = DataObject()
        obj.url = magento_url
        obj.auth_type = 'none'
        magento_request = ApiRequest(obj)

        aws_bucket_name = 'aheadworks_cdn'
        aws_swagger_web_api_doc_path = 'swagger_web_api_doc/'
        magento_app_code_path_on_server = f"{magento_path_on_server}/app/code"
        tmp_dir_m2_modules = '/var/tmp/m2_modules'

        parent_module_name = self.magento_manager.get_module_name(path_to_module)
        if not os.path.isfile(f"{path_to_module}/etc/webapi.xml"):
            return f'Skip Web API doc generation: file etc/webapi.xml has been not found for module {parent_module_name}'

        parsed_url = urlparse(magento_url)

        try:
            list_of_module_paths = self.magento_manager.download_modules_from_git(path_to_module, tmp_dir_m2_modules)
            self.magento_manager.upload_modules_to_server_by_ssh(
                list_of_module_paths['dir_app_code'],
                magento_app_code_path_on_server,
                parsed_url.hostname,
                ssh_port,
                ssh_user,
                ssh_pass
            )
            module_names = ','.join(list_of_module_paths['module_names'])
            magento_request_url = '/generate_web_api_json.php?module_names={}'.format(module_names)
            # workaround, for generate magento cache
            magento_request.get(location=magento_request_url)
            # here we generate json with cache from previous step
            swagger_json = magento_request.get(location=magento_request_url)

            try:
                json.loads(swagger_json)
            except Exception as error:
                print(f"Invalid respone from Swagger:\n{swagger_json}\n\n")
                raise Exception(error)

            s3_result = self.aws_s3.Bucket(aws_bucket_name).put_object(
                Key=f"{aws_swagger_web_api_doc_path}{parent_module_name.lower()}_latest.json",
                Body=swagger_json,
                ACL='public-read'
            )
        except Exception as error:
            raise Exception(error)
        finally:
            os.system(f"rm -rf {tmp_dir_m2_modules}")

        result = f'Web Api Doc Path: https://media.aheadworks.com/{s3_result.key}\n'
        result += f'Magento Request Url: {magento_request_url}\n'
        return result

    def build_ecommerce_pack(self, bitbucket_workspace, bitbucket_repo_slug):
        ComposerManager.init_extra_repos()

        build_dir = os.getcwd()
        artifacts_dir = "/build_archives"
        path = f"{build_dir}/app/code/Aheadworks"
        self.file_manager.create_empty_dir("app/code/Aheadworks")
        self.file_manager.create_empty_dir(artifacts_dir)

        with open('./composer.json') as f:
            composer = json.load(f)
        core_module_name = composer['name']
        module_dependencies = self.magento_manager.get_module_dependencies('./')
        ComposerManager.require_magento_module(core_module_name, composer['version'])

        # @todo use self.magento_manager.download_modules_from_git(path_to_module, tmp_dir_m2_modules)
        magento_module_info = dict()
        artifacts = list()
        # Preinstall suggested modules with composer into /var/www/html/vendor/aheadworks
        for full_module_name, module_item in module_dependencies.items():
            if self.magento_manager.is_suggested_module(build_dir, full_module_name):
                ComposerManager.require_magento_module(full_module_name)

        # full_module_name          vendor/module-name
        # module_name               module-name
        # magento_package_name      ModuleName
        for full_module_name, module_item in module_dependencies.items():
            module_name = module_item['module_name']
            composer_install_path = f"/var/www/html/vendor/aheadworks/{module_name}"
            path_to_composer = f"{composer_install_path}/composer.json"
            path_to_registration = f"{composer_install_path}/registration.php"
            if not os.path.isfile(path_to_composer) or not os.path.isfile(path_to_registration):
                os.system(f"ls {composer_install_path}")
                raise Exception(f"Can not build module {full_module_name}: composer.json or/and registration.php is missing")

            magento_package_name = self._get_magento_package_name(composer_install_path)
            magento_module_info[full_module_name] = magento_package_name
            module_path = f"{path}/{magento_package_name}"

            print(f"Building {full_module_name} as {module_name} in {module_path}...")
            shutil.copytree(composer_install_path, module_path)

            print("\t\u21E8 Cleanup filesystem...")
            self.file_manager.remove_files_and_dirs_ignore_case(
                module_path,
                ['bitbucket-pipelines.yml', 'readme.md', '.gitignore'],
                ['.git']
            )

            print("\t\u21E8 Writing license headers...")
            self.file_manager.add_license_to_php_files(
                module_path,
                magento_package_name,
                self.json_parser.get_variable_from_file('version', f"{module_path}/composer.json")
            )
            os.system(f"echo See https://aheadworks.com/end-user-license-agreement/ >> {module_path}/license.txt")
            artifacts.append(
                self._pack_module(module_path, f"{artifacts_dir}/{module_name}.zip")
            )

        # Now build store packages
        for filename in self._get_store_package_names(magento_module_info[core_module_name], composer['version']):
            artifacts.append(
                self._pack_module(build_dir, f"{artifacts_dir}/{filename}", "app")
            )

        self._upload_artifacts(bitbucket_workspace, bitbucket_repo_slug, artifacts)

    def build_mm_pack(self, bitbucket_workspace, bitbucket_repo_slug):
        # basically marketplace pack is a ecommerce pack where all modules have -subscription postfix in their names
        # so we assume that the ecommerce pack has been built so far
        prebuilt_packages_dir = "/build_archives"
        working_dir = "/tmp/unpack"
        package_dir = "/tmp/packages"
        self.file_manager.create_empty_dir(working_dir)
        self.file_manager.create_empty_dir(package_dir)
        os.system(f"ls -l {prebuilt_packages_dir}")
        # Extract all packages first
        for filename in os.listdir(prebuilt_packages_dir):
            package_fullpath = os.path.join(prebuilt_packages_dir, filename)
            # TODO: process *.zip only
            if os.path.isfile(package_fullpath):
                sources_dir = os.path.join(working_dir, filename)
                print(package_fullpath)
                os.system(f"unzip -q {package_fullpath} -d {sources_dir}")

        # Now gather module names
        module_names = []
        composer_files = self.file_manager.find_all('composer.json', working_dir)
        print(composer_files) # debug
        for composer_file in composer_files:
            package_name = self.json_parser.get_variable_from_file('name', composer_file)
            if not package_name in module_names:
                module_names.append(package_name)

        print(module_names) # debug
        # Replace all found modules names from "vendor/module_name" to "vendor/module_name-subscription"
        # across all composer.json (this includes requires/suggests sections and possibly even more)
        for composer_file in composer_files:
            self._replace_dependencies(composer_file, module_names, "subscription")

        # Create '-subscription' packages
        for originalname in os.listdir(working_dir):
            sources_fullpath = os.path.join(working_dir, originalname)

            split_name, split_extension = os.path.splitext(originalname)
            target_filename = f"{split_name}-subscription{split_extension}"
            target_fullpath = os.path.join(package_dir, target_filename)
            print(target_fullpath) # debug
            self._pack_module(sources_fullpath, target_fullpath)

        # Copy newly built -subscription packages to /build_archives
        os.system(f"cp {package_dir}/* {prebuilt_packages_dir}")

    # Sample metapackage product: https://bitbucket.org/awm2ext/b2b-suite/
    def build_metapackage_pack(self, bitbucket_workspace, bitbucket_repo_slug, artifacts_dir):
        upload_env = f"REPO_URL={os.getenv('AW_COMPOSER_API_URL')} REPO_LOGIN={os.getenv('AW_COMPOSER_API_LOGIN')} REPO_TOKEN={os.getenv('AW_COMPOSER_API_PASSWORD')}"

        # prepare FS
        build_dir = os.getcwd()
        if not artifacts_dir:
            artifacts_dir = "/build_archives"
        path = f"{build_dir}/app/code/Aheadworks"
        self.file_manager.create_empty_dir(artifacts_dir)
        self.file_manager.create_empty_dir(path)

        # Get dependent packages list
        with open("./composer.json") as f:
            composer = json.load(f)

        # Install dependent packages via Composer first
        for full_module_name in composer["require"]:
            ComposerManager.require_magento_module(full_module_name)

        artifacts = []
        for full_module_name in composer["require"]:
            module = full_module_name.split('/')[1]
            metapackage_module_name = f"{module}-{bitbucket_repo_slug}"

            composer_install_path = f"/var/www/html/vendor/aheadworks/{module}"
            magento_package_name = self._get_magento_package_name(composer_install_path)
            module_path = f"{path}/{magento_package_name}"

            print(f"Building {full_module_name} as {module} in {module_path}...")
            shutil.copytree(composer_install_path, module_path)

            print("\t\u21E8 Cleanup filesystem...")
            self.file_manager.remove_files_and_dirs_ignore_case(
                module_path,
                ['bitbucket-pipelines.yml', 'readme.md', '.gitignore'],
                ['.git']
            )

            with open(f"{module_path}/composer.json") as f:
                module_composer = json.load(f)

            print("\t\u21E8 Writing license headers...")
            self.file_manager.add_license_to_php_files(
                module_path,
                magento_package_name,
                module_composer['version']
            )
            os.system(f"echo See https://aheadworks.com/end-user-license-agreement/ >> {module_path}/license.txt")

            old_name = module_composer['name']
            new_name = f"{module_composer['name']}-{bitbucket_repo_slug}"

            print(f"\t\u21E8 Renaming {old_name} to {new_name}...")
            module_composer['name'] = new_name
            with open(f"{module_path}/composer.json", 'w') as f:
                json.dump(module_composer, f, indent=4)

            self._replace_dependencies(f"{module_path}/composer.json", composer["require"], bitbucket_repo_slug)

            zip_package_path = f"{artifacts_dir}/{metapackage_module_name}.zip"
            artifacts.append(
                self._pack_module(module_path, zip_package_path)
            )

            upload_as = f"{metapackage_module_name}-{module_composer['version']}.zip"
            os.system(
                f"echo '{upload_env} python3 -m aheadworks_composer_manager send-package --filename={upload_as} {zip_package_path}' >> {artifacts_dir}/upload")
            # ??? Why we upload to Composer server in this noobish manner instead of using API ???
            composer_server_base_path = f"{os.getenv('AW_COMPOSER_PACKAGES_ROOT')}/aheadworks"
            os.system(
                f"ssh -T {os.getenv('AW_COMPOSER_SSH_URL')} 'mkdir -p {composer_server_base_path}/{metapackage_module_name}'")
            os.system(
                f"scp {zip_package_path} {os.getenv('AW_COMPOSER_SSH_URL')}:{composer_server_base_path}/{metapackage_module_name}/{upload_as}")

        core_version = composer['version']
        with open('composer.json') as f:
            lines = f.readlines()
            for line in f.readlines():
                lines.append(line)
            result_lines = list()
            for line in lines:
                if line.find('aheadworks/module') != -1:
                    line = line.split(':')[0][:-1] + f'-{bitbucket_repo_slug}"' + f":{line.split(':')[1]}"
                result_lines.append(line)
        with open('/tmp/composer.json', 'w') as f:
            for line in result_lines:
                f.write(line)

        # build marketplace package
        artifacts.append(
            self._pack_module("/tmp", f"{artifacts_dir}/{bitbucket_repo_slug}.zip", "composer.json")
        )

        # store package
        for filename in self._get_store_package_names(bitbucket_repo_slug, core_version):
            artifacts.append(
                self._pack_module(build_dir, f"{artifacts_dir}/{filename}", "app")
            )

        self._upload_artifacts(bitbucket_workspace, bitbucket_repo_slug, artifacts)

    def _replace_dependencies(self, path_to_composer, module_names, slug):
        fin = open(path_to_composer, "rt")
        data = fin.read()
        fin.close()

        for module_name in module_names:
            # we intentionally quote vendor/package_name string to be "vendor/package_name".
            # otherwise vendor/package_name_subname could be renamed
            # into vendor/package_name-subscription_subname
            # instead of vendor/package_name_subname-subscription
            data = data.replace(f'"{module_name}"', f'"{module_name}-{slug}"')

        fin = open(path_to_composer, "wt")
        fin.write(data)
        fin.close()

    def _pack_module(self, source_module_path, target_package_path, base_dir="."):
        print(f"\t\u21E8 Packing {source_module_path} into {target_package_path}\n")
        base_name, extension = os.path.splitext(target_package_path)
        shutil.make_archive(base_name, extension.lstrip('.'), source_module_path, base_dir)
        return target_package_path

    def _get_store_package_names(self, magento_package_name, version):
        return [
            f"aw_m2_{magento_package_name}-{version}.community_edition.zip",
            f"aw_m2_{magento_package_name}-{version}.enterprise_edition.zip"
        ]

    def _get_magento_package_name(self, module_path):
        with open(f"{module_path}/registration.php") as reg:
            l = reg.readlines()
            for line in l:
                if line.find("Aheadworks_") != -1:
                    m = re.search("Aheadworks_([^\"']+)", line)
                    return m.group(1)
        return ""

    def _upload_artifacts(self, bitbucket_workspace, bitbucket_repo_slug, filepath):
        print(f"\nUploading artifacts\n{filepath}...\n")
        bitbucket_manager = BitbucketApiManager(
            BitbucketConfig(
                bitbucket_workspace=bitbucket_workspace,
                bitbucket_repo_slug=bitbucket_repo_slug
            )
        )
        print(bitbucket_manager.upload_artifacts(filepath))



#!/usr/bin/env python3
from aheadworks_core.api.composer_manager import ComposerManager
from aheadworks_core.model.cd import Cd as cd
import subprocess
import click
from os.path import join as _
import os
import re
import json
import pathlib
import time

def remove_listeners(path):
    with open(path) as phpunit_config:
        data = phpunit_config.read()
        reg = re.compile("<listeners.*</listeners>", re.S)
        data = re.sub(reg, '', data)
    with open(path, 'w') as phpunit_config:
        phpunit_config.write(data)

def di_compile():
    proc = subprocess.Popen(['php', '/var/www/html/bin/magento', 'module:enable', '--all'])
    proc.communicate()
    ec3 = proc.returncode
    proc = subprocess.Popen(['php', '/var/www/html/bin/magento', 'setup:di:compile'])
    ec4 = proc.returncode

    if ec3 or ec4:
        print("Failed to di:compile")

def install(path):
    """
    Install extension(s) to path from path or zip
    """
    repo_type = 'path'

    click.echo("Installing from %s" % path)

    with open(path / 'composer.json') as f:
        composer = json.load(f)
        repo_name = re.sub(r'[^a-z0-9_]', '_', composer['name'])

    with cd(BASIC_PATH):
        f = open('auth.json.sample')
        composer_auth = json.load(f)
        f.close()

        proc = subprocess.Popen(['php', '-d', 'memory_limit=4G', '/usr/local/bin/composer', 'config', '-g', 'repositories.aheadworks', 'composer', 'https://composer.do.staging-box.net'])
        proc.communicate()

        # composer_auth["http-basic"]["repo.magento.com"]["username"] = "ca6f970bb96d25614c90874edc90f42b"
        # composer_auth["http-basic"]["repo.magento.com"]["password"] = "74ce69e7288fc3232fb3b038410e9ae6"
        composer_auth["http-basic"]["repo.magento.com"]["username"] = "bf08744e4b4b3aee1d54fcd7cd56194a"
        composer_auth["http-basic"]["repo.magento.com"]["password"] = "f5b232eab5158a4597ecb00f8cacdf4a"
        os.system('touch auth.json')
        with open('auth.json', 'w') as file:
            json.dump(composer_auth, file, indent=2)
        os.system('composer config repositories.magento composer https://repo.magento.com/')
        proc = subprocess.Popen(['composer', 'config', 'repositories.' + repo_name, repo_type, path])
        proc.communicate()
        ec1 = proc.returncode

        proc = subprocess.Popen(['php', '-d', 'memory_limit=4G', '/usr/local/bin/composer', 'require', '--prefer-dist', '{e[name]}:{e[version]}'.format(e=composer)])
        proc.communicate()
        ec2 = proc.returncode

        if ec1 or ec2:
            raise click.ClickException("Failed to install extension")
    result_path = BASIC_PATH / 'vendor' / composer['name']
    return result_path


@click.group()
def cli():
    click.echo("Removing phpunit listeners")
    remove_listeners(BASIC_PATH / 'dev' / 'tests' / 'static' / 'phpunit.xml.dist')
    remove_listeners(BASIC_PATH / 'dev' / 'tests' / 'unit' / 'phpunit.xml.dist')


@cli.command()
@click.option('--severity', default=10, help='Severity level.')
@click.option('--report', default="junit", help='Report type.', type=click.Choice(["full", "xml", "checkstyle", "csv",
                                                                             "json", "junit", "emacs", "source",
                                                                             "summary", "diff", "svnblame", "gitblame",
                                                                             "hgblame", "notifysend"]))
@click.argument('path', type=click.Path(exists=True))
@click.argument('report_file', type=click.Path(), required=False)
def eqp(severity, report, path, report_file):
    """Run EQP tests for path"""

    proc = subprocess.Popen([_('/magento-coding-standard', 'vendor/bin/phpcs'), path, '--standard=Magento2',
                             '--severity='+str(severity), '--extensions=php,phtml', '--report='+report],
                            stdout=subprocess.PIPE
                            )
    stdout, stderr = proc.communicate()

    if report_file:
        with open(report_file, 'wb') as fp:
            fp.write(stdout)
    else:
        click.echo(stdout)
    exit(proc.returncode)


@cli.command()
@click.option('--report', default="junit", help='Report type.', type=click.Choice(["junit"]))
@click.argument('path', type=click.Path(exists=True))
@click.argument('report_file', type=click.Path(), required=False)
def unit(report, path, report_file):
    """Run unit tests for extension at path"""

    path = pathlib.Path(path)
    path = install(path)
    di_compile()

    options = [
        _(BASIC_PATH, 'vendor/bin/phpunit'),
        '--configuration',  _(BASIC_PATH, 'dev/tests/unit/phpunit.xml.dist')
    ]

    if report_file:
        options += ['--log-%s' % report, report_file]

    proc = subprocess.Popen(options + [_(path, 'Test/Unit')])
    proc.communicate()

    if not report_file:
        exit(proc.returncode)

    exit(proc.returncode)


@cli.command()
@click.option('--report', default="junit", help='Report type.', type=click.Choice(["junit"]))
@click.argument('path', type=click.Path(exists=True))
@click.argument('report_path', type=click.Path(), required=False)
def static(report, path, report_path):
    """
    Run static tests against path
    :param report:
    :param path:
    :param report_file:
    :return:
    """

    path = pathlib.Path(path)

    phpcs_report_file = path / 'test-results' / 'test-phpcs.xml'

    #path = install(path)

    path_changed_files = BASIC_PATH / 'dev' / 'tests' / 'static' / 'testsuite' / 'Magento' / 'Test' / 'Php' / '_files' / 'whitelist' / 'common.txt'
    path_blacklist = BASIC_PATH / 'dev' / 'tests' / 'static' / 'testsuite' / 'Magento' / 'Test' / 'Php' / '_files' / 'phpstan' / 'blacklist' / 'common.txt'

    options = [os.path.join(BASIC_PATH, 'vendor/bin/phpunit'), '--configuration',
               BASIC_PATH / 'dev/tests/static/phpunit.xml.dist']

    output_base = report_path or os.environ.get('RESULTS_DIR', '/results')

    with open(os.path.join(BASIC_PATH, 'dev/tests/static/phpunit.xml.dist')) as phpunit_config:
        suites = {}
        reg = re.compile('<testsuite[^>]+name="([^"]+)"')
        for line in phpunit_config:
            try:
                suite = re.search(reg, line).groups()[0]
                suites[suite.lower().replace(' ', '_')] = suite
            except (IndexError, AttributeError):
                pass

    # Collect php iles
    f = open(path_blacklist, 'r')
    lines = f.readlines()
    f.close()

    newLines = []
    for line in lines:
        linePath = line
        linePath = linePath.rstrip()
        if line[0] != '#' and os.path.isfile(BASIC_PATH / linePath):
            newLines.append(line)

    lines = newLines

    with open(path_blacklist, 'w') as fp:
        fp.writelines(lines)
    f = open(path_changed_files, 'r')
    lines = f.readlines()
    f.close()
    lines = lines[0:2]
    with open(path_changed_files, 'w') as fp:
        fp.writelines(lines)
        for root, dirs, files in os.walk(path):
            fp.writelines([os.path.relpath(os.path.abspath(os.path.join(root, f)), BASIC_PATH) + '\n' for f in files if
                           os.path.splitext(f)[1] in (
                               '.php',
                               '.phtml'
                           )])

    exit_code = 0
    print(suites.items())
    for fname, name in suites.items():

        outfile = os.path.join(output_base, fname + '.xml')

        if re.search(re.compile('integrity'), fname):
            continue
        if re.search(re.compile('less'), fname):
            continue
        if re.search(re.compile('html'), fname):
            continue

        args = options + ['--testsuite=%s' % name]
        args += ['--log-%s' % report, outfile]
        proc = subprocess.Popen(args)
        proc.communicate()

        # Remove copy-paste results from report
        with open(os.path.join(output_base, fname + '.xml')) as f:
            data = f.read()
            data = re.sub('<testcase[^>]+name="testCopyPaste".+</testcase>', '', data,
                          flags=re.MULTILINE and re.DOTALL)

        with open(outfile, 'w') as f:
            f.write(data)

        exit_code = proc.returncode or exit_code

    # test code style
    severity = '9'
    if 'SEVERITY' in os.environ:
        if not os.getenv('SEVERITY') == '':
            severity = str(os.getenv('SEVERITY'))
    proc = subprocess.Popen(
        ['/var/www/html/vendor/bin/phpcs', path, '--standard=Magento2', '--extensions=php,phtml', '--severity={}'.format(severity),
         '--report=' + report],
        stdout=subprocess.PIPE
        )
    stdout, stderr = proc.communicate()

    if phpcs_report_file:
        with open(phpcs_report_file, 'wb') as fp:
            fp.write(stdout)
    else:
        click.echo(stdout)
    if proc.returncode != 0:
        exit_code = proc.returncode

    exit(exit_code)


@cli.command()
@click.option('--report', default="junit", help='Report type.', type=click.Choice(["junit"]))
@click.argument('path', type=click.Path(exists=True))
@click.argument('report_file', type=click.Path(), required=False)
def install_magento(report, path, report_file):
    """Run install magento with extension at path"""

    global output
    path = pathlib.Path(path)
    path = install(path)

    try:
        output = subprocess.run(
            ['sh', "/tmp/create-install-dump.sh", '--withoutdump'],
            check=True,
            text=True
        )
    except subprocess.CalledProcessError as e:
        os.system("mkdir logs1")
        os.system("mysqldump -uroot -proot magento >> logs1/dump.sql")
        os.system(f"cp /var/www/html/var/log/* logs1")
        os.system("cp /var/www/html/app/etc/* logs1")
        exit(e.returncode)

    exit(output.returncode)

@cli.command()
@click.argument('path', type=click.Path(exists=True))
def validate_m2_package(path):
    """
    Test marketplace package
    :param path:
    :return:
    """
    #proc = subprocess.Popen(['php', '-f', '/usr/local/bin/validate_m2_package.php', path])
    #proc.communicate()
    #exit(proc.returncode)

if __name__ == '__main__':
    BASIC_PATH = pathlib.Path(os.environ.get('MAGENTO_ROOT', '/var/www/html'))
    ComposerManager.init_extra_repos()
    cli()
